@@ -8,6 +8,7 @@ from tempfile import NamedTemporaryFile
 from collections import namedtuple
 import click
 import yaml
+import requests
 
 import boto
 from boto.ec2 import connect_to_region
@@ -35,6 +36,7 @@ from distutils.spawn import find_executable
 import pkg_resources
 VERSION = pkg_resources.get_distribution("myria-cluster").version
 CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
+USE_PREBUILT_AMI = False
 
 SCRIPT_NAME =  os.path.basename(sys.argv[0])
 # this is necessary because pip loses executable permissions and ansible requires scripts to be executable
@@ -64,16 +66,17 @@ DEFAULTS = dict(
 )
 
 DEFAULT_AMI_IDS = {
-    'us-east-1': "ami-fce3c696",
-    'us-west-2': "ami-9abea4fb",
-    'us-west-1': "ami-06116566",
-    'eu-west-1': "ami-f95ef58a",
-    'eu-central-1': "ami-87564feb",
-    'ap-southeast-1': "ami-25c00c46",
-    'ap-northeast-1': "ami-a21529cc",
-    'ap-southeast-2': "ami-6c14310f",
-    'ap-northeast-2': "ami-09dc1267",
-    'sa-east-1': "ami-0fb83963"
+    'us-west-2': "ami-d75392b7",
+    'us-east-1': "ami-f9219bee",
+    'us-west-1': "ami-0799de67",
+    'eu-west-1': "ami-91c6a3e2",
+    'eu-central-1': "ami-4d52b922",
+    'ap-northeast-1': "ami-7917e418",
+    'ap-northeast-2': "ami-7f844e11",
+    'ap-southeast-1': "ami-27b76a44",
+    'ap-southeast-2': "ami-c1371ca2",
+    'ap-south-1': "ami-d094febf",
+    'sa-east-1': "ami-b6099cda",
 }
 
 ANSIBLE_GLOBAL_VARS = yaml.load(file(ANSIBLE_GLOBAL_VARS_PATH, 'r'))
@@ -83,15 +86,17 @@ class Options(object):
     """
     Options class to replace Ansible OptParser
     """
-    def __init__(self, subset=None, syntax=False, listhosts=False, listtasks=False, listtags=False, module_path=None,
-                 forks=MAX_CONCURRENT_TASKS, connection='smart', remote_user=None, private_key_file=None,
-                 ssh_common_args=None, sftp_extra_args=None, scp_extra_args=None, ssh_extra_args=None,
-                 become=False, become_method='sudo', become_user='root', verbosity=0, check=False):
+    def __init__(self, subset=None, syntax=False, listhosts=False, listtasks=False, listtags=False,
+                 tags=None, module_path=None, forks=MAX_CONCURRENT_TASKS, connection='smart',
+                 remote_user=None, private_key_file=None, ssh_common_args=None, sftp_extra_args=None,
+                 scp_extra_args=None, ssh_extra_args=None, become=False, become_method='sudo',
+                 become_user='root', verbosity=0, check=False):
         self.subset = subset
         self.syntax = syntax
         self.listhosts = listhosts
         self.listtasks = listtasks
         self.listtags = listtags
+        self.tags = tags
         self.module_path = module_path
         self.forks = forks
         self.connection = connection
@@ -110,7 +115,7 @@ class Options(object):
 
 class Runner(object):
 
-    def __init__(self, hostnames, playbook, private_key_file, run_data, become_pass=None,
+    def __init__(self, hostnames, playbook, private_key_file, run_data, tags=None, become_pass=None,
                  verbosity=0, callback=None, subset_pattern=None):
 
         self.hostnames = hostnames
@@ -118,7 +123,7 @@ class Runner(object):
         self.playbook = os.path.join(playbooks_dir, playbook)
         self.run_data = run_data
 
-        self.options = Options(subset=subset_pattern, private_key_file=private_key_file, verbosity=verbosity)
+        self.options = Options(tags=tags, subset=subset_pattern, private_key_file=private_key_file, verbosity=verbosity)
 
         self.display = Display()
         self.display.verbosity = verbosity
@@ -379,7 +384,12 @@ def default_key_file_from_key_pair(ctx, param, value):
 
 
 def default_ami_id_from_region(ctx, param, value):
-    return value if value is not None else DEFAULT_AMI_IDS[ctx.params['region']]
+    global USE_PREBUILT_AMI
+    if value is None:
+        USE_PREBUILT_AMI = True
+        return DEFAULT_AMI_IDS[ctx.params['region']]
+    else:
+        return value
 
 
 def validate_subnet_id(ctx, param, value):
@@ -540,10 +550,9 @@ Cluster '{cluster_name}' already exists in the '{region}' region. If you wish to
         run_data=extra_vars,
         verbosity=kwargs['verbose'],
         callback=CallbackModule(kwargs['verbose'], failed_hosts))
-    local_runner = Runner(**playbook_args)
     success = False
     try:
-        success = local_runner.run()
+        success = Runner(**playbook_args).run()
     except Exception as e:
         if kwargs['verbose'] > 0:
             click.echo(e)
@@ -560,16 +569,33 @@ Cluster '{cluster_name}' already exists in the '{region}' region. If you wish to
             pass # best-effort
         sys.exit(1)
 
+    # poll instances for status until all are reachable
+    group = get_security_group_for_cluster(cluster_name, kwargs['region'], profile=kwargs['profile'], vpc_id=vpc_id)
+    instance_ids = [instance.id for instance in group.instances()]
+    while True:
+        ec2 = connect_to_region(kwargs['region'], profile_name=kwargs['profile'])
+        statuses = ec2.get_all_instance_status(instance_ids=instance_ids)
+        for status in statuses:
+            if status.system_status.details['reachability'] != "passed":
+                if kwargs['verbose'] > 0:
+                    click.echo("Not all instances reachable, waiting 60 seconds...")
+                sleep(60)
+                break
+        else:
+            break
+
     # run remote playbook to provision EC2 instances
     retries = 0
     retry_hosts_pattern = None
     # TODO: exponential backoff for unreachable hosts?
     while True:
         retry_hosts = set()
-        playbook_args.update(hostnames=INVENTORY_SCRIPT_PATH, playbook="remote.yml", callback=CallbackModule(kwargs['verbose'], retry_hosts), subset_pattern=retry_hosts_pattern)
-        remote_runner = Runner(**playbook_args)
+        playbook_args.update(hostnames=INVENTORY_SCRIPT_PATH, playbook="remote.yml",
+            callback=CallbackModule(kwargs['verbose'], retry_hosts), subset_pattern=retry_hosts_pattern)
+        if USE_PREBUILT_AMI:
+            playbook_args.update(tags=['configure'])
         try:
-            success = remote_runner.run()
+            success = Runner(**playbook_args).run()
         except Exception as e:
             if kwargs['verbose'] > 0:
                 click.echo(e)
@@ -598,6 +624,32 @@ Cluster '{cluster_name}' already exists in the '{region}' region. If you wish to
         click.echo("Couldn't resolve coordinator public DNS, exiting (not destroying cluster)")
         sys.exit(1)
 
+    # wait for Myria REST service to be available and all workers online
+    workers_url = "http://%(host)s:%(port)d/workers" % dict(host=coordinator_public_hostname, port=ANSIBLE_GLOBAL_VARS['myria_rest_port'])
+    while True:
+        try:
+            workers_resp = requests.get(workers_url)
+        except requests.ConnectionError:
+            if kwargs['verbose'] > 0:
+                click.echo("Myria service unavailable, waiting 60 seconds...")
+        else:
+            if workers_resp.status_code == requests.codes.ok:
+                workers = workers_resp.json()
+                workers_alive_resp = requests.get(workers_url + "/alive")
+                workers_alive = workers_alive_resp.json()
+                if len(workers_alive) == len(workers):
+                    break
+                else:
+                    if kwargs['verbose'] > 0:
+                        click.echo("Not all Myria workers online (%d/%d), waiting 60 seconds..." % (
+                            len(workers_alive), len(workers)))
+            else:
+                click.echo("Error response from Myria service (status code %d):\n%s\n\nDestroying cluster..." % (
+                    workers_resp.status_code, workers_resp.text))
+                terminate_cluster(cluster_name, kwargs['region'], profile=kwargs['profile'], vpc_id=vpc_id)
+                sys.exit(1)
+        sleep(60)
+
     click.echo("""
 Your new Myria cluster '{cluster_name}' has been launched on Amazon EC2 in the '{region}' region.
 
